package kr.co.vividnext.sodalive.admin.charge

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.can.QCan.can1
import kr.co.vividnext.sodalive.can.charge.ChargeStatus
import kr.co.vividnext.sodalive.can.charge.QCharge.charge
import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.can.payment.QPayment.payment
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
class AdminChargeStatusQueryRepository(private val queryFactory: JPAQueryFactory) {
    fun getChargeStatus(startDate: LocalDateTime, endDate: LocalDateTime): List<GetChargeStatusQueryDto> {
        val formattedDate = Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                charge.createdAt,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d"
        )

        return queryFactory
            .select(
                QGetChargeStatusQueryDto(
                    formattedDate,
                    payment.price.sum(),
                    can1.price.sum(),
                    payment.id.count(),
                    payment.paymentGateway
                )
            )
            .from(payment)
            .innerJoin(payment.charge, charge)
            .leftJoin(charge.can, can1)
            .where(
                charge.createdAt.goe(startDate)
                    .and(charge.createdAt.loe(endDate))
                    .and(charge.status.eq(ChargeStatus.CHARGE))
                    .and(payment.status.eq(PaymentStatus.COMPLETE))
            )
            .groupBy(formattedDate, payment.paymentGateway)
            .orderBy(formattedDate.desc())
            .fetch()
    }

    fun getChargeStatusDetail(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        paymentGateway: PaymentGateway
    ): List<GetChargeStatusDetailQueryDto> {
        val formattedDate = Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                charge.createdAt,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d %H:%i:%s"
        )

        return queryFactory
            .select(
                QGetChargeStatusDetailQueryDto(
                    member.id,
                    member.nickname,
                    payment.method.coalesce(""),
                    payment.price,
                    can1.price,
                    formattedDate
                )
            )
            .from(charge)
            .innerJoin(charge.member, member)
            .innerJoin(charge.payment, payment)
            .leftJoin(charge.can, can1)
            .where(
                charge.createdAt.goe(startDate)
                    .and(charge.createdAt.loe(endDate))
                    .and(charge.status.eq(ChargeStatus.CHARGE))
                    .and(payment.status.eq(PaymentStatus.COMPLETE))
                    .and(payment.paymentGateway.eq(paymentGateway))
            )
            .orderBy(formattedDate.desc())
            .fetch()
    }
}
