package kr.co.vividnext.sodalive.admin.live

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.recommend.QRecommendLiveCreatorBanner.recommendLiveCreatorBanner
import kr.co.vividnext.sodalive.live.recommend.RecommendLiveCreatorBanner
import kr.co.vividnext.sodalive.live.room.LiveRoom
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Repository

@Repository
class AdminLiveRoomQueryRepository(private val queryFactory: JPAQueryFactory) {
    fun getLiveRoomList(): List<LiveRoom> {
        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(liveRoom.isActive.isTrue)
            .orderBy(liveRoom.channelName.desc(), liveRoom.beginDateTime.asc())
            .fetch()
    }

    fun getRecommendCreatorTotalCount(): Int {
        return queryFactory
            .select(recommendLiveCreatorBanner.id)
            .from(recommendLiveCreatorBanner)
            .fetch()
            .size
    }

    fun getRecommendCreatorList(pageable: Pageable): List<RecommendLiveCreatorBanner> {
        return queryFactory
            .selectFrom(recommendLiveCreatorBanner)
            .offset(pageable.offset)
            .limit(pageable.pageSize.toLong())
            .orderBy(recommendLiveCreatorBanner.orders.asc(), recommendLiveCreatorBanner.id.desc())
            .fetch()
    }
}
