package kr.co.vividnext.sodalive.live.room.visit

import kr.co.vividnext.sodalive.live.room.LiveRoom
import kr.co.vividnext.sodalive.live.room.detail.GetRoomDetailUser
import kr.co.vividnext.sodalive.member.Member
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.time.LocalDateTime

@Service
@Transactional(readOnly = true)
class LiveRoomVisitService(
    private val repository: LiveRoomVisitRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    @Transactional
    fun roomVisit(room: LiveRoom, member: Member) {
        var roomVisit = repository.findByRoomIdAndMemberId(room.id!!, member.id!!)
        if (roomVisit == null) {
            roomVisit = LiveRoomVisit()
            roomVisit.member = member
            roomVisit.room = room
        } else {
            roomVisit.updatedAt = LocalDateTime.now()
        }

        repository.save(roomVisit)
    }

    fun getRecentVisitRoomUsers(memberId: Long): List<GetRoomDetailUser> {
        val roomVisit = repository.findFirstByMemberIdOrderByUpdatedAtDesc(memberId)
            ?: return emptyList()

        return repository.getRecentVisitRoomUsers(roomVisit.room!!.id!!, memberId)
            .asSequence()
            .map { GetRoomDetailUser(it, cloudFrontHost) }
            .toList()
    }
}
