package kr.co.vividnext.sodalive.member.auth

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.auth.QAuth.auth
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
interface AuthRepository : JpaRepository<Auth, Long>, AuthQueryRepository

interface AuthQueryRepository {
    fun getOldestCreatedAtByDi(di: String): LocalDateTime
    fun getMemberIdsByDi(di: String): List<Long>
}

class AuthQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : AuthQueryRepository {
    override fun getOldestCreatedAtByDi(di: String): LocalDateTime {
        return queryFactory
            .select(auth.createdAt)
            .from(auth)
            .where(auth.di.eq(di))
            .orderBy(auth.createdAt.asc())
            .fetchFirst()
    }

    override fun getMemberIdsByDi(di: String): List<Long> {
        return queryFactory
            .select(member.id)
            .from(member)
            .leftJoin(member.auth, auth)
            .where(auth.di.eq(di))
            .fetch()
    }
}
